%%
% Eisma, Y. B., Van Vliet, S. B., Nederveen, A., & De Winter, J. C. F. (2024). Assessing the influence of visual stimulus properties on steady-state visually evoked potentials and pupil diameter. Biomedical Physics & Engineering Express. https://doi.org/10.1088/2057-1976/ad865d
% 1 Department of Cognitive Robotics, Delft University of Technology
% 2 Department of Radiology and Nuclear Medicine, Amsterdam UMC Location AMC, Amsterdam
% Corresponding author: j.c.f.dewinter@tudelft.nl

clear all;close all;clc %#ok<CLALL>
ini % load brightness data of the video clips
L=10000; % number of samples of stimulus (4 seconds)
Fs=2500; % sampling frequency

% Experiment 1, also named Experiment 1x1, displays a single stimulus at a
% time. The stimulus here will vary by shape, size, color, and frequency.

% In Experiment 2, also named Experiment 2x2, 4 stimuli are presented simultaneously.
% One stimulus of these 4 stimuli is the target stimulus. This stimulus is displayed at the same frequencies used in Experiment 1x1.
% The other stimuli are shown at frequencies close to the target frequency.

readraw_exp=0;

% if 1, read EEG data of Experiment 1, and save as .mat file
% if 2, read EEG data of Experiment 2, and save as .mat file
% if 3, process pupil data and perform discrete Fourier transforms of all trials and all EEG channels

if readraw_exp==1 % read EEG data of Experiment 1, and save as MAT file
    for p=1:6 % loop over 6 participants
        load(['EEG data\pp' num2str(p) '_e1.mat'])
        d=EEG.data;
        time=EEG.times;
        clear E S
        for i=1:length(EEG.event)
            E(i,1)=EEG.event(i).latency;
            E(i,2)=EEG.event(i).duration;
            if ~isempty(EEG.event(i).bvtime)
                E(i,3)=EEG.event(i).bvtime;
            end
            E(i,4)=EEG.event(i).bvmknum;
            E(i,5)=EEG.event(i).urevent;
            S{i}=EEG.event(i).type;
        end
        save(['OA' num2str(p) '_1.mat'],'d','time','E','S','-v7.3')
    end
end

if readraw_exp==2  % read EEG data of Experiment 2, and save as MAT file
    for p=1:6 % loop over 6 participants
        load(['EEG data\pp' num2str(p) '_e2.mat'])
        d=EEG.data;
        time=EEG.times;
        clear E S

        for i=1:length(EEG.event)
            E(i,1)=EEG.event(i).latency;
            E(i,2)=EEG.event(i).duration;
            if ~isempty(EEG.event(i).bvtime)
                E(i,3)=EEG.event(i).bvtime;
            end
            E(i,4)=EEG.event(i).bvmknum;
            E(i,5)=EEG.event(i).urevent;
            S{i}=EEG.event(i).type;
        end
        save(['OA' num2str(p) '_2.mat'],'d','time','E','S','-v7.3')
    end
end
if readraw_exp==3 % load eye-tracking data and perform FFT; save as .mat file
    %% Experiment 1: Load eye-tracking data
    LP=7930;
    PTS = NaN(6,328,LP); % pre-allocate matrix (6 participants x 328 trials)
    for p=1:6 % loop over 6 participants    
        filename=['Eye-tracking data\P' num2str(p) '_Experiment1.mat'];
        load(filename)
        for i=1:328 % loop over 328 trials of participant p

            disp(['Perform filtering of pupil size - Experiment 1, Participant: ' num2str(p) ', Trial: ' num2str(i)]) % display participant number and trial number
            idt=find(x(:,20)==i);
            idt=idt(1:LP);
            xx=x(idt,2); % gaze x data of trial
            yy=x(idt,3); % gaze y data of trial
            pp=x(idt,4); % pupil size data of trial
            pp1=x(idt,10); % pupil size of left eye
            pp2=x(idt,13); % pupil size of right eye

            fr=x(idt,17); % frame number
            try
                [p1_medfilt,p2_medfilt,is_NaN1,is_NaN2] = filterpupil(pp1,pp2);
                md=(p1_medfilt+p2_medfilt)/2;
                PTS(p,i,:)=100*(md-md(1))./md(1);
            catch error
            end
        end
    end
    %% Experiment 2: Load eye-tracking data
    PTS2 = NaN(6,436,LP); % pre-allocate matrix (6 participants x 436 trials)
    X2 = NaN(6,436,LP); % pre-allocate matrix (6 participants x 436 trials
    Y2 = NaN(6,436,LP); % pre-allocate matrix (6 participants x 436 trials)
    FF2 = NaN(6,436,19,6);
    for p=1:6 % loop over 6 participants
        filename=['Eye-tracking data\P' num2str(p) '_Experiment2.mat'];
        load(filename)
        for i=1:436 % loop over 436 trials of participant p

            disp(['Perform filtering of pupil size - Experiment 2, Participant: ' num2str(p) ', Trial: ' num2str(i)]) % display participant number and trial number
            idt=find(x(:,20)==i);
            idt=idt(1:LP);
            xx=x(idt,2); % gaze x data of trial
            yy=x(idt,3); % gaze y data of trial
            pp1=x(idt,10); % pupil size of left eye
            pp2=x(idt,13); % pupil size of right eye

            try
                [p1_medfilt,p2_medfilt,is_NaN1,is_NaN2] = filterpupil(pp1,pp2);
                md=(p1_medfilt+p2_medfilt)/2;
                PTS2(p,i,:)=100*(md-md(1))./md(1);
            catch error
            end
        end

    end

    % 1	    {'TRIAL_INDEX'          }
    % 2	    {'AVERAGE_GAZE_X'       }
    % 3	    {'AVERAGE_GAZE_Y'       }
    % 4	    {'AVERAGE_PUPIL_SIZE'   }
    % 5	    {'IP_DURATION'          }
    % 6	    {'IP_END_TIME'          }
    % 7	    {'IP_START_TIME'        }
    % 8	    {'LEFT_GAZE_X'          }
    % 9	    {'LEFT_GAZE_Y'          }
    % 10	    {'LEFT_PUPIL_SIZE'      }
    % 11	    {'RIGHT_GAZE_X'         }
    % 12	    {'RIGHT_GAZE_Y'         }
    % 13	    {'RIGHT_PUPIL_SIZE'     }
    % 14	    {'SAMPLE_INDEX'         }
    % 15	    {'TIMESTAMP'            }
    % 16	    {'TRIAL_START_TIME'     }
    % 17	    {'VIDEO_FRAME_INDEX'    }
    % 18	    {'Block_variable'       }
    % 19	    {'Total_elapsed_time'   }
    % 20	    {'Trial_Index_'         }
    % 21	    {'begin_time_trial'     }
    % 22	    {'displayed_frame_count'}
    % 23	    {'dropped_frame_count'  }
    % 24	    {'end_time_trial'       }
    % 25	    {'frequency'            }
    % 26	    {'pixel_surface'        }
    % 27	shape
    % 28	color

    %% Perform a FFT of all trials - Experiment 1
    F = NaN(6,328,63,5001); % pre-allocate matrix with FFT results (6 participants x 328 trials x 63 electrodes x 5001 frequency points)

    for p=1:6 % loop over 6 participants
        load(['EEG data\OA' num2str(p) '_1.mat'])
        % contains data for participant
        % d: EEG data
        % time: time vector (equal length to d)
        % S: vector consisting of trial types
        % E: matrix of trial info (latency and trial number)

        valid_indices=ismember(S,{'S  1','S  2'}); % only retain experimental trials
        E=E(valid_indices,:);
        S=S(valid_indices); % vector that contains "S 1" or "S 2"

        clear T
        ids=E(2:2:end,1); % indices of starts of trials ("S 2" are the trials we are interested in)

        for i=1:length(ids) % loop over 328 trials of participant p
            disp(['Perform FFT - Experiment 1, Participant: ' num2str(p) ', Trial: ' num2str(i)]) % display participant number and trial number
            dt=d(:,ids(i):ids(i)+L-1); % data of trial (ensure there is 4 seconds of data; take 10000 samples at 2500 Hz)

            for c = 1:size(d,1) % loop over the channels
                Y = fft(dt(c,:));
                P2 = abs(Y/L);
                P1 = P2(1:L/2+1); % 1:5000
                P1(2:end-1) = 2*P1(2:end-1);
                F(p,i,c,:) = P1; % perform a FFT for the voltage signal of channel C
            end
        end
    end
    %% Perform a FFT of all trials - Experiment 2
    F2 = NaN(6,436,63,5001); % pre-allocate matrix with FFT results (6 participants x 436 trials x 63 trials x 5001 frequency points)
    % In Experiment 2, there are 144 combinations (as compared to 108 in Experiment 1)
    % 4 positions (LT, RT, LB, RB) x 4 frequencies x 3 sizes x 3 colors (only squares)
    for p=1:6 % loop over 6 participants
        load(['EEG data\OA' num2str(p) '_2.mat'])
        % contains data for participant
        % d: EEG data
        % time: time vector (equal length to d)
        % S: vector consisting of trial types
        % E: matrix of trial info (latency and trial number)

        valid_indices=ismember(S,{'S  3','S  4'}); % only retain experimental trials
        E=E(valid_indices,:);
        S=S(valid_indices); % vector that contains "S 3" or "S 4"

        clear T
        ids=E(2:2:end,1); % indices of starts of trials ("S 4" are the trials we are interested in)

        for i=1:length(ids) % loop over 436 trials of participant p
            disp(['Perform FFT - Experiment 2, Participant: ' num2str(p) ', Trial: ' num2str(i)]) % display participant number and trial number
            dt=d(:,ids(i):ids(i)+L-1); % data of trial (ensure there is 4 seconds of data; take 10000 samples at 2500 Hz)

            for c = 1:size(d,1) % loop over the channels
                Y = fft(dt(c,:));
                P2 = abs(Y/L);
                P1 = P2(1:L/2+1); % 1:5000
                P1(2:end-1) = 2*P1(2:end-1);
                F2(p,i,c,:) = P1; % perform a FFT for the voltage signal of channel C
            end
        end
    end
    F=F(:,:,:,1:321); % only up to 75 Hz, to save memory
    F2=F2(:,:,:,1:321); % only up to 75 Hz, to save memory
    save('Pupil_and_fft.mat','PTS','PTS2','F','F2','-v7.3')
else
    load Pupil_and_fft.mat
end

%% Preprocess all data
% #     Name      Phys. Chn.    Resolution / Unit   Low Cutoff [s]   High Cutoff [Hz]   Notch [Hz]    Gradient         Offset
% 1     Fp1         1          0.0488281 µV             DC              690              Off
% 2     Fz          2          0.0488281 µV             DC              690              Off
% 3     F3          3          0.0488281 µV             DC              690              Off
% 4     F7          4          0.0488281 µV             DC              690              Off
% 5     FT9         5          0.0488281 µV             DC              690              Off
% 6     FC5         6          0.0488281 µV             DC              690              Off
% 7     FC1         7          0.0488281 µV             DC              690              Off
% 8     C3          8          0.0488281 µV             DC              690              Off
% 9     T7          9          0.0488281 µV             DC              690              Off
% 10    TP9         10         0.0488281 µV             DC              690              Off
% 11    CP5         11         0.0488281 µV             DC              690              Off
% 12    CP1         12         0.0488281 µV             DC              690              Off
% 13    Pz          13         0.0488281 µV             DC              690              Off
% 14    P3          14         0.0488281 µV             DC              690              Off
% 15    P7          15         0.0488281 µV             DC              690              Off
% 16    O1          16         0.0488281 µV             DC              690              Off
% 17    Oz          17         0.0488281 µV             DC              690              Off
% 18    O2          18         0.0488281 µV             DC              690              Off
% 19    P4          19         0.0488281 µV             DC              690              Off
% 20    P8          20         0.0488281 µV             DC              690              Off
% 21    TP10        21         0.0488281 µV             DC              690              Off
% 22    CP6         22         0.0488281 µV             DC              690              Off
% 23    CP2         23         0.0488281 µV             DC              690              Off
% 24    Cz          24         0.0488281 µV             DC              690              Off
% 25    C4          25         0.0488281 µV             DC              690              Off
% 26    T8          26         0.0488281 µV             DC              690              Off
% 27    FT10        27         0.0488281 µV             DC              690              Off
% 28    FC6         28         0.0488281 µV             DC              690              Off
% 29    FC2         29         0.0488281 µV             DC              690              Off
% 30    F4          30         0.0488281 µV             DC              690              Off
% 31    F8          31         0.0488281 µV             DC              690              Off
% 32    AF7         33         0.0488281 µV             DC              690              Off
% 33    AF3         34         0.0488281 µV             DC              690              Off
% 34    AFz         35         0.0488281 µV             DC              690              Off
% 35    F1          36         0.0488281 µV             DC              690              Off
% 36    F5          37         0.0488281 µV             DC              690              Off
% 37    FT7         38         0.0488281 µV             DC              690              Off
% 38    FC3         39         0.0488281 µV             DC              690              Off
% 39    C1          40         0.0488281 µV             DC              690              Off
% 40    C5          41         0.0488281 µV             DC              690              Off
% 41    TP7         42         0.0488281 µV             DC              690              Off
% 42    CP3         43         0.0488281 µV             DC              690              Off
% 43    P1          44         0.0488281 µV             DC              690              Off
% 44    P5          45         0.0488281 µV             DC              690              Off
% 45    PO7         46         0.0488281 µV             DC              690              Off
% 46    PO3         47         0.0488281 µV             DC              690              Off
% 47    POz         48         0.0488281 µV             DC              690              Off
% 48    PO4         49         0.0488281 µV             DC              690              Off
% 49    PO8         50         0.0488281 µV             DC              690              Off
% 50    P6          51         0.0488281 µV             DC              690              Off
% 51    P2          52         0.0488281 µV             DC              690              Off
% 52    CPz         53         0.0488281 µV             DC              690              Off
% 53    CP4         54         0.0488281 µV             DC              690              Off
% 54    TP8         55         0.0488281 µV             DC              690              Off
% 55    C6          56         0.0488281 µV             DC              690              Off
% 56    C2          57         0.0488281 µV             DC              690              Off
% 57    FC4         58         0.0488281 µV             DC              690              Off
% 58    FT8         59         0.0488281 µV             DC              690              Off
% 59    F6          60         0.0488281 µV             DC              690              Off
% 60    AF8         61         0.0488281 µV             DC              690              Off
% 61    AF4         62         0.0488281 µV             DC              690              Off
% 62    F2          63         0.0488281 µV             DC              690              Off
% 63    Iz          64         0.0488281 µV             DC              690              Off

all_electrodes=0;
if all_electrodes==0
    ELECTRODES = 17;
else
    ELECTRODES=1:63;
end
SSSS=NaN(length(ELECTRODES),2);

SNRall1=NaN(length(ELECTRODES),81,8); % pre-allocate matrix for storing all SNR values, Experiment 1 (electrodes x 81 conditions x 8 stimulus & response frequency combinations)
SNRall2=NaN(length(ELECTRODES),108,8); % pre-allocate matrix for storing all SNR values, Experiment 2 (electrodes x 108 conditions x 8 stimulus & response frequency combinations)

for e = 1:length(ELECTRODES)
    disp(['Electrode number: ' num2str(ELECTRODES(e))])
    FTS=squeeze(mean(F(:,:,ELECTRODES(e),:),3));
    FTS2=squeeze(mean(F2(:,:,ELECTRODES(e),:),3));

    % Experiment 1: Create CDV matrix; a 324 x 5 matrix with an overview of condition numbers
    cdv=NaN(324,5);
    for i1=1:3 % block index (1, 2, or 3). A block consists of 108 trials
        for i2=1:4 % 4 frequencies
            for i3=1:3 % surface area (10000, 20000, 30000)
                for i4=1:3 % shape index (square, circle, triangle)
                    for i5=1:3 % color index (white, green, red)
                        cdv((i1-1)*108 + ...
                            (i2-1)*27 + ...
                            (i3-1)*9 + ...
                            (i4-1)*3 + ...
                            i5,1:5) = [i1 i2 i3 i4 i5]; % block index, frequency, surface area (size), shape, color
                    end
                end
            end
        end
    end
    % Experiment 2: Create CDV2 matrix; a 432 x 5 matrix with an overview of condition numbers
    cdv2=NaN(432,5);
    for i1=1:3 % block index (1, 2, or 3). A block consists of 144 trials
        for i2=1:4 % 4 frequencies
            for i3=1:3 % surface area (10000, 20000, 30000)
                for i4=1:4 % location index (LT, RT, LB, RB)
                    for i5=1:3 % color index (white, green, red)
                        cdv2((i1-1)*144 + ...
                            (i2-1)*36 + ...
                            (i3-1)*12 + ...
                            (i4-1)*3 + ...
                            i5,1:5) = [i1 i2 i3 i4 i5]; % block index, frequency, surface area (size), location, color
                    end
                end
            end
        end
    end

    % Experiment 1: Sort the FFT results in FTSa matrix
    f = Fs*(0:(L/2))/L; % vector of frequencies (0, 0.25, 0.50, ... 1249.75, 1250)
    FTSa=NaN(6,324,321); % pre-allocate with NaNs
    PTSa=NaN(6,324,size(PTS,3)); % pre-allocate with NaNs
    Block=1:7;
    Freq=[8 13 19 25];
    Surface=[10000 20000 30000];
    Shape={'s','c','t'};
    Color={'w','g','r'};

    % Note that 4 of the trials are practice trials (Block 1 consists of 4 trials). Hence, the number of experimental trials is 324, not 328
    % The experimental trials, which cover 3 times the 108 combinations, are divided up into 6 sets (blocks), each of 54 trials, with a break in between

    for p=1:6 % loop over 6 participants
        warning off all
        T=tdfread(['Conditions\p' num2str(p) '_1.txt']);
        warning on all

        bi=ismember(T.Block_variable,2:3)+2*ismember(T.Block_variable,4:5)+3*ismember(T.Block_variable,6:7); % 328 x 1 vector with block index (1, 2 or 3)
        [~,fi] = ismember(T.frequency,Freq); % 328 x 1 vector with frequency number
        [~,ai] = ismember(T.pixel_surface,Surface); % 328 x 1 vector surface area number
        [~,si] = ismember(T.shape(:,1),Shape); % 328 x 1 vector shape number
        [~,ci] = ismember(T.color_mode(:,1),Color); % 328 x 1 vector color number

        cd = (bi-1)*108 + (fi-1)*27 + (ai-1)*9 + (si-1)*3 + ci;
        cd(cd<0) = NaN; % set practice trials to NaN
        % cd is a 328 x 1 vector with condition numbers, from 1 to 324

        for c=1:324 % loop over all condition numbers
            d=squeeze(FTS(p,cd==c,:));
            if ~isempty(d)
                FTSa(p,c,:)=d; % sort the FFT results into a 6 x 324 x 5001 matrix, with the conditions corresponding to the cdv matrix
            end

            d=squeeze(PTS(p,cd==c,:));
            if ~isempty(d)
                PTSa(p,c,:)=d; % sort the pupil diameter results into a 6 x 324 x 7930 matrix, with the conditions corresponding to the cdv matrix
            end

        end
    end
    % Experiment 2: Sort the FFT2 results in FTS2a matrix
    FTS2a=NaN(6,432,321); % pre-allocate with NaNs
    PTS2a=NaN(6,432,size(PTS,3)); % pre-allocate with NaNs

    % Note that 4 of the trials are practice trials (Block 1 consists of 4 trials). Hence, the number of experimental trials is 432, not 436
    % The experimental trials, which cover 3 times the 144 combinations, are divided up into 6 sets (blocks), with a break in between

    for p=1:6 % loop over 6 participants
        warning off all
        T=tdfread(['Conditions\p' num2str(p) '_2.txt']);
        warning on all

        bi=ismember(T.Block_variable,2:3)+2*ismember(T.Block_variable,4:5)+3*ismember(T.Block_variable,6:7); % 436 x 1 vector with block index (1, 2 or 3)
        [~,fi] = ismember(T.frequency_2,Freq); % 436 x 1 vector with frequency number
        [~,ai] = ismember(T.pixel_surface_2,Surface); % 436 x 1 vector surface area number
        li = contains(cellstr(T.photo_clip_inter_trial_2),'480_270') + 2*contains(cellstr(T.photo_clip_inter_trial_2),'1440_270') + 3*contains(cellstr(T.photo_clip_inter_trial_2),'480_810') + 4*contains(cellstr(T.photo_clip_inter_trial_2),'1440_810');
        [~,ci] = ismember(T.color_mode_2(:,1),Color); % 436 x 1 vector color number

        cd=(bi-1)*144 + (fi-1)*36 + (ai-1)*12 + (li-1)*3 + ci;
        cd(cd<0) = NaN; % set practice trials to NaN
        % cd is a 436 x 1 vector with condition numbers, from 1 to 432

        for c=1:432 % loop over all condition numbers
            d=squeeze(FTS2(p,cd==c,:)); %
            if ~isempty(d)
                FTS2a(p,c,:)=d; % sort the FFT results into a 6 x 432 x 5001 matrix, with the conditions corresponding to the cdv matrix
            end

            d=squeeze(PTS2(p,cd==c,:)); %
            if ~isempty(d)
                PTS2a(p,c,:)=d; % sort the pupil diameter results into a 6 x 432 x 7930 matrix, with the conditions corresponding to the cdv matrix
            end

        end
    end

    % Calculate baseline FFT per participant - Experiment 1
    fft1=FTSa(:,cdv(:,2)==1,:);fft1(:,:,ismember(f,[7.5 7.75 8 8.25 8.5        15.5 15.75 16 16.25 16.5]))=NaN;fft1=squeeze(mean(fft1,2)); % 8, 16 removed
    fft2=FTSa(:,cdv(:,2)==2,:);fft2(:,:,ismember(f,[12.5 12.75 13  13.25 13.5  25.5 25.75 26 26.25 26.5]))=NaN;fft2=squeeze(mean(fft2,2)); % 13, 26 removed
    fft3=FTSa(:,cdv(:,2)==3,:);fft3(:,:,ismember(f,[18.5 18.75 19 19.25 19.5   37.5 37.75 38 38.25 38.5]))=NaN;fft3=squeeze(mean(fft3,2)); % 19, 38 removed
    fft4=FTSa(:,cdv(:,2)==4,:);fft4(:,:,ismember(f,[24.5 24.75 25 25.25 25.5]))=NaN;fft4=squeeze(mean(fft4,2)); % 25 removed
    fft_all(1,:,:)=fft1;
    fft_all(2,:,:)=fft2;
    fft_all(3,:,:)=fft3;
    fft_all(4,:,:)=fft4;
    fft_exp1_baseline=squeeze(mean(fft_all,1,'omitnan'));

    % Calculate baseline FFT per participant - Experiment 2
    fft1=FTS2a(:,cdv2(:,2)==1,:);fft1(:,:,ismember(f,[7.5 7.75 8 8.25 8.5        15.5 15.75 16 16.25 16.5]))=NaN;fft1=squeeze(mean(fft1,2)); % 8, 16 removed
    fft2=FTS2a(:,cdv2(:,2)==2,:);fft2(:,:,ismember(f,[12.5 12.75 13  13.25 13.5  25.5 25.75 26 26.25 26.5]))=NaN;fft2=squeeze(mean(fft2,2)); % 13, 26 removed
    fft3=FTS2a(:,cdv2(:,2)==3,:);fft3(:,:,ismember(f,[18.5 18.75 19 19.25 19.5   37.5 37.75 38 38.25 38.5]))=NaN;fft3=squeeze(mean(fft3,2)); % 19, 38 removed
    fft4=FTS2a(:,cdv2(:,2)==4,:);fft4(:,:,ismember(f,[24.5 24.75 25 25.25 25.5]))=NaN;fft4=squeeze(mean(fft4,2)); % 25 removed
    fft_all(1,:,:)=fft1;
    fft_all(2,:,:)=fft2;
    fft_all(3,:,:)=fft3;
    fft_all(4,:,:)=fft4;
    fft_exp2_baseline=squeeze(mean(fft_all,1,'omitnan'));

    bl1=NaN(6,324,321);
    bl2=NaN(6,432,321);
    for p=1:6
        bl1(p,:,:)=repmat(fft_exp1_baseline(p,:)',1,324)';
        bl2(p,:,:)=repmat(fft_exp2_baseline(p,:)',1,432)';
    end
    FTSa_blc  = FTSa./bl1; % baseline corrected
    FTS2a_blc = FTS2a./bl2;

    % Experiment 1: Calculate SNR at target frequencies
    MPF=NaN(8,4,6,81); % 8 selected frequencies x 4 stimulus frequencies x  6 participants x 81 trials per stimulus frequency
    RF = [8 13 19 25 16 26 38 50]; % 8 selected frequencies (stimulus frequencies, and stimulus frequencies x 2)
    ks = 1;
    k = 16;
    for i = 1:length(RF) % loop over selected response frequencies (8, 13, 19, 25, 16, 26, 38, 50 Hz)
        [~,Fid] = min(abs(f-RF(i))); % find index of frequency RF(i)

        v1s = FTSa_blc(:,cdv(:,2)==1,Fid); % stimulus frequency 8 Hz (6 x 81 matrix)
        v2s = FTSa_blc(:,cdv(:,2)==2,Fid); % stimulus frequency 13 Hz (6 x 81 matrix)
        v3s = FTSa_blc(:,cdv(:,2)==3,Fid); % stimulus frequency 19 Hz (6 x 81 matrix)
        v4s = FTSa_blc(:,cdv(:,2)==4,Fid); % stimulus frequency 25 Hz (6 x 81 matrix)

        v1n = mean(FTSa_blc(:,cdv(:,2)==1,[Fid-k:Fid-8 Fid+8:Fid+k]),3); % stimulus frequency 8 Hz (6 x 81 matrix)
        v2n = mean(FTSa_blc(:,cdv(:,2)==2,[Fid-k:Fid-8 Fid+8:Fid+k]),3); % stimulus frequency 13 Hz (6 x 81 matrix)
        v3n = mean(FTSa_blc(:,cdv(:,2)==3,[Fid-k:Fid-8 Fid+8:Fid+k]),3); % stimulus frequency 19 Hz (6 x 81 matrix)
        v4n = mean(FTSa_blc(:,cdv(:,2)==4,[Fid-k:Fid-8 Fid+8:Fid+k]),3); % stimulus frequency 25 Hz (6 x 81 matrix)

        MPF(i,1,1:size(v1s,1),1:size(v1s,2))=v1s./v1n; % SNR for target frequency divided by surrounding frequences (8 response frequencies x 4 stimulus frequencies x 6 participants x 81 trials)
        MPF(i,2,1:size(v2s,1),1:size(v2s,2))=v2s./v2n;
        MPF(i,3,1:size(v3s,1),1:size(v3s,2))=v3s./v3n;
        MPF(i,4,1:size(v4s,1),1:size(v4s,2))=v4s./v4n;
    end

    % Experiment 2: Calculate SNR at target frequencies
    MPF2=NaN(8,4,6,108); % 8 selected frequencies x 4 stimulus frequencies x  6 participants x 108 trials per stimulus frequency

    for i = 1:length(RF) % loop over selected response frequencies (8, 13, 19, 25, 16, 26, 38, 50 Hz)

        [~,Fid] = min(abs(f-RF(i))); % find index of frequency F(i)

        v1s = FTS2a_blc(:,cdv2(:,2)==1,Fid); % stimulus frequency 8 Hz (6 x 108 matrix)
        v2s = FTS2a_blc(:,cdv2(:,2)==2,Fid); % stimulus frequency 13 Hz (6 x 108 matrix)
        v3s = FTS2a_blc(:,cdv2(:,2)==3,Fid); % stimulus frequency 19 Hz (6 x 108 matrix)
        v4s = FTS2a_blc(:,cdv2(:,2)==4,Fid); % stimulus frequency 25 Hz (6 x 108 matrix)

        v1n = mean(FTS2a_blc(:,cdv2(:,2)==1,[Fid-k:Fid-8 Fid+8:Fid+k]),3); % stimulus frequency 8 Hz (6 x 108 matrix)
        v2n = mean(FTS2a_blc(:,cdv2(:,2)==2,[Fid-k:Fid-8 Fid+8:Fid+k]),3); % stimulus frequency 13 Hz (6 x 108 matrix)
        v3n = mean(FTS2a_blc(:,cdv2(:,2)==3,[Fid-k:Fid-8 Fid+8:Fid+k]),3); % stimulus frequency 19 Hz (6 x 108 matrix)
        v4n = mean(FTS2a_blc(:,cdv2(:,2)==4,[Fid-k:Fid-8 Fid+8:Fid+k]),3); % stimulus frequency 25 Hz (6 x 108 matrix)

        MPF2(i,1,1:size(v1s,1),1:size(v1s,2))=v1s./v1n; % target frequency divided by surrounding frequences (8 response frequencies x 4 stimulus frequencies x 6 participants x 108 trials)
        MPF2(i,2,1:size(v2s,1),1:size(v2s,2))=v2s./v2n;
        MPF2(i,3,1:size(v3s,1),1:size(v3s,2))=v3s./v3n;
        MPF2(i,4,1:size(v4s,1),1:size(v4s,2))=v4s./v4n;
    end

    % Experiment 1: classification
    Classification1 = NaN(4,6,81); % pre-allocate matrix with classification results (0 = incorrect, 1 = correct): 4 stimulus frequencies x 6 participants x 81 trials
    for i=1:4 % loop over 4 stimulus frequencies
        for i2=1:6 % loop over 6 participants
            for i3=1:81 % loop over 81 trials
                mpf = MPF(1:8,i,i2,i3); % SNR for the 8 response frequencies (8, 13, 19, 25, 16, 26, 38, 50 Hz), 4 stimulus frequencies (8, 13, 19, 25 Hz), 6 participants, and 81 trials
                mpf(1:3)=(mpf(1:3)+mpf(5:7))/2; % take the average of first and second harmonic
                mpf(5:8)=[];
                [~,h] = max(mpf); % index of response frequency with highest value

                if h==i
                    Classification1(i,i2,i3) = 1;
                else
                    Classification1(i,i2,i3) = 0;
                end
            end
        end
    end
    disp(round([mean(mean(Classification1(1,:,:)))  mean(mean(Classification1(2,:,:)))   mean(mean(Classification1(3,:,:)))   mean(mean(Classification1(4,:,:)))  ],2))

    % Experiment 2: classification
    Classification2 = NaN(4,6,108); % pre-allocate matrix with classification results (0 = incorrect, 1 = correct): 4 stimulus frequencies x 6 participants x 108 trials
    for i=1:4 % loop over 4 stimulus frequencies
        for i2=1:6 % loop over 6 participants
            for i3=1:108 % loop over 108 trials
                mpf = MPF2(1:8,i,i2,i3); % SNR for the 8 response frequencies (8, 13, 19, 25, 16, 26, 38, 50 Hz), 4 stimulus frequencies (8, 13, 19, 25 Hz), 6 participants, and 108 trials
                mpf(1:3)=(mpf(1:3)+mpf(5:7))/2; % take the average of first and second harmonic
                mpf(5:8)=[];
                [~,h] = max(mpf); % index of response frequency with highest value

                if h==i
                    Classification2(i,i2,i3) = 1;
                else
                    Classification2(i,i2,i3) = 0;
                end
            end
        end
    end
    disp(round([mean(mean(Classification2(1,:,:)))  mean(mean(Classification2(2,:,:)))   mean(mean(Classification2(3,:,:)))   mean(mean(Classification2(4,:,:)))  ],2))

    % Experiment 1: Compute SNR
    DIF=NaN(8,81,6);
    for i=1:8 % loop over stimulus frequencies and second harmonic
        if i<=4 % first harmonic
            target=squeeze(MPF(i,i,:,:))';
            DIF(i,:,:)=target; % FFT magnitude at target frequency, for trials of stimulus frequency
        else % second harmonic
            target=squeeze(MPF(i,i-4,:,:))';
            DIF(i,:,:)=target; % FFT magnitude at target frequency, for trials of stimulus frequency
        end
    end

    Conditions = cdv(cdv(:,2)==1,[3 4 5]); % 3: surface area (10000, 20000, 30000), 4: shape index (square, circle, triangle), 5: color index (white, green, red)

    % surface area (10000, 20000, 30000)
    % shape index (square, circle, triangle)
    % color index (white, green, red)
    SNR=mean(DIF,3)';  % take mean DIF across the 6 participants
    clear c p
    [~,p(1,:),c(1,1:2,1:8)]=ttest(SNR(Conditions(:,1)==1,:)); % size (surface area)
    [~,p(2,:),c(2,1:2,1:8)]=ttest(SNR(Conditions(:,1)==2,:));
    [~,p(3,:),c(3,1:2,1:8)]=ttest(SNR(Conditions(:,1)==3,:));
    [~,p(4,:),c(4,1:2,1:8)]=ttest(SNR(Conditions(:,2)==1,:)); % shape
    [~,p(5,:),c(5,1:2,1:8)]=ttest(SNR(Conditions(:,2)==2,:));
    [~,p(6,:),c(6,1:2,1:8)]=ttest(SNR(Conditions(:,2)==3,:));
    [~,p(7,:),c(7,1:2,1:8)]=ttest(SNR(Conditions(:,3)==1,:)); % color
    [~,p(8,:),c(8,1:2,1:8)]=ttest(SNR(Conditions(:,3)==2,:));
    [~,p(9,:),c(9,1:2,1:8)]=ttest(SNR(Conditions(:,3)==3,:));
    ci=squeeze((c(:,2,:)-c(:,1,:))/2);

    % Experiment 2: Compute SNR and create bar graph
    DIF2=NaN(4,108,6);
    for i=1:8 % loop over 4 stimulus frequencies
        if i<=4 % first harmonic
            target=squeeze(MPF2(i,i,:,:))'; % all FFT data at target frequency, for trials of stimulus frequency
            DIF2(i,:,:)=target;
        else % second harmonic
            target=squeeze(MPF2(i,i-4,:,:))'; % all FFT data at target frequency, for trials of stimulus frequency
            DIF2(i,:,:)=target;
        end
    end

    Conditions2=cdv2(cdv2(:,2)==1,[3 4 5]); % 3: surface area (10000, 20000, 30000), 4: location index (LT, RT, LB, RB), 5: color index (white, green, red)

    SNR2=mean(DIF2(:,:,:),3)';  % take mean DIF across the 6 participants
    clear c2

    [~,~,c2(1,1:2,1:8)]=ttest(SNR2(Conditions2(:,1)==1,:));
    [~,~,c2(2,1:2,1:8)]=ttest(SNR2(Conditions2(:,1)==2,:));
    [~,~,c2(3,1:2,1:8)]=ttest(SNR2(Conditions2(:,1)==3,:));
    [~,~,c2(4,1:2,1:8)]=ttest(SNR2(Conditions2(:,2)==1,:));
    [~,~,c2(5,1:2,1:8)]=ttest(SNR2(Conditions2(:,2)==2,:));
    [~,~,c2(6,1:2,1:8)]=ttest(SNR2(Conditions2(:,2)==3,:));
    [~,~,c2(7,1:2,1:8)]=ttest(SNR2(Conditions2(:,2)==4,:));
    [~,~,c2(8,1:2,1:8)]=ttest(SNR2(Conditions2(:,3)==1,:));
    [~,~,c2(9,1:2,1:8)]=ttest(SNR2(Conditions2(:,3)==2,:));
    [~,~,c2(10,1:2,1:8)]=ttest(SNR2(Conditions2(:,3)==3,:));
    ci2=squeeze((c2(:,2,:)-c2(:,1,:))/2);

    SNRall1(e,:,:)=SNR;
    SNRall2(e,:,:)=SNR2;
end
%%
if all_electrodes==1
    disp('SNR values for all 63 electrodes, Experiment 1 and Experiment 2')
    meanSNRelectrodes = [mean(mean(SNRall1(:,:,1:4),3),2) mean(mean(SNRall2(:,:,1:4),3),2)];
    [~,b]=sortrows(meanSNRelectrodes,1,'descend'); % sort mean SNR in descending order for Experiment 1
    disp(meanSNRelectrodes(b,:))
    [~,electrode_names]=xlsread('electrodes.xlsx'); %#ok<XLSRD>
    figure
    h = bar(meanSNRelectrodes(b,:));
    set(h(1), 'FaceColor', [0 0 0]); % Color for Experiment 1
    set(h(2), 'FaceColor', [255 165 0]/255); % Color for Experiment 2
    grid on;ax=gca; ax.LineWidth=1.5;
    set(gca,'xtick',1:63)
    set(gca,'xticklabels',electrode_names(b))
    set(gca,'ylim',[1 2.1])
    set(gca,'tickdir','out')
    legend('Experiment 1×1','Experiment 2×2')
    xlabel('Electrode')
    ylabel('Mean SNR')
    set(findall(gcf,'-property','FontName'),'FontName','Arial','Fontsize',22)
    set(gca,'LooseInset', [0.01 0.01 0.01 0.01])
    return
end

%%
SNR = squeeze(SNRall1(:,:,:)); % use SNR values for a single electrode, Experiment 1
SNR2 = squeeze(SNRall2(:,:,:)); % use SNR values for a single electrode, Experiment 2

%% Figure 3. Experiment 1: FFT figure of EEG data
f = Fs*(0:(L/2))/L; % vector of frequencies (0, 0.25, 0.50, ... 1249.75, 1250)
f=f(1:321);
P=1:6;

figure
set(gcf,'renderer','painters')
temp=squeeze(mean(FTSa(P,cdv(:,2)==1,:),1)); % mean FFT for stimulus frequency of 8 Hz
plot(f,mean(temp),'color',[1 0.5 0],'Linewidth',4);hold on

temp=squeeze(mean(FTSa(P,cdv(:,2)==2,:),1)); % mean FFT for stimulus frequency of 13 Hz
plot(f,mean(temp),'color',[0.5 1 0],'Linewidth',4);

temp=squeeze(mean(FTSa(P,cdv(:,2)==3,:),1)); % mean FFT for stimulus frequency of 19 Hz
plot(f,mean(temp),'color',[0 0 1],'Linewidth',4);

temp=squeeze(mean(FTSa(P,cdv(:,2)==4,:),1)); % mean FFT for stimulus frequency of 25 Hz
plot(f,mean(temp),'color',[0 0 0],'Linewidth',4);

set(gca,'xlim',[1 76],'ylim',[0 2.5],'xtick',0:2:100)
grid on;ax=gca; ax.LineWidth=1.5;
legend('8 Hz','13 Hz','19 Hz','25 Hz')
title('Experiment 1x1')
xlabel('Frequency (Hz)')
ylabel('Magnitude')
set(findall(gcf,'-property','FontName'),'FontName','Arial','Fontsize',30)
set(gca,'LooseInset', [0.01 0.01 0.01 0.01])

% Figure 4. Experiment 2: FFT figure of EEG data
figure
set(gcf,'renderer','painters')
P=1:6;
temp=squeeze(mean(FTS2a(P,cdv2(:,2)==1,:),1)); % mean FFT for stimulus frequency of 8 Hz
plot(f,mean(temp),'color',[1 0.5 0],'Linewidth',4);hold on

temp=squeeze(mean(FTS2a(P,cdv2(:,2)==2,:),1)); % mean FFT for stimulus frequency of 13 Hz
plot(f,mean(temp),'color',[0.5 1 0],'Linewidth',4);

temp=squeeze(mean(FTS2a(P,cdv2(:,2)==3,:),1)); % mean FFT for stimulus frequency of 19 Hz
plot(f,mean(temp),'color',[0 0 1],'Linewidth',4);

temp=squeeze(mean(FTS2a(P,cdv2(:,2)==4,:),1)); % mean FFT for stimulus frequency of 25 Hz
plot(f,mean(temp),'color',[0 0 0],'Linewidth',4);
set(gca,'xlim',[1 76],'ylim',[0 2.5],'xtick',0:2:100)
grid on;ax=gca; ax.LineWidth=1.5;
legend('8 Hz','13 Hz','19 Hz','25 Hz')
xlabel('Frequency (Hz)')
ylabel('Magnitude')
title('Experiment 2x2')
set(findall(gcf,'-property','FontName'),'FontName','Arial','Fontsize',30)
set(gca,'LooseInset', [0.01 0.01 0.01 0.01])

%% Figure 5 and 6: bar graphs
ads=[mean(SNR(Conditions(:,1)==1,1:7),2) mean(SNR(Conditions(:,1)==2,1:7),2) mean(SNR(Conditions(:,1)==3,1:7),2)]; % SNR for the three sizes (use 7 of 8 response frequences, i.e., not 50 Hz)
[~,~,adsc]=ttest(ads);
disp('Experiment 1: Mean SNR for the 3 sizes (n = 27, or 324/(4 frequencies x 3 shapes)')
disp(round(mean(ads),3))
disp('Experiment 1: 95% confidence intervals of SNRs for the 3 sizes')
disp(round(adsc,3))

adshape=[mean(SNR(Conditions(:,2)==1,1:7),2) mean(SNR(Conditions(:,2)==2,1:7),2) mean(SNR(Conditions(:,2)==3,1:7),2)]; % SNR for the three shapes (use 7 of 8 response frequences, i.e., not 50 Hz)
[~,~,adshapec]=ttest(adshape);
disp('Experiment 1: Mean SNR for the 3 shapes (square, circle, triangle) (n = 27, or 324/(4 frequencies x 3 shapes)')
disp(round(mean(adshape),3))
disp('Experiment 1: 95% confidence intervals of SNRs for the 3 shapes (square, circle, triangle)')
disp(round(adshapec,3))

figure
set(gcf,'renderer','painters')
b=bar(squeeze(mean(c,2,'omitnan'))');hold on
title('Experiment 1x1')
b(1).FaceColor=[0.9 0.9 0.9];
b(2).FaceColor=[0.7 0.7 0.7];
b(3).FaceColor=[0.5 0.5 0.5];
b(4).FaceColor=[0 1 1];
b(5).FaceColor=[127 255 212]/255;
b(6).FaceColor=[0 206 209]/255;
b(7).FaceColor=[1 1 1 ];
b(8).FaceColor=[0 1 0 ];
b(9).FaceColor=[1 0 0 ];
legend('Small','Medium','Large','Square','Circle','Triangle','White','Green','Red','autoupdate','off','location','eastoutside');
set(gca,'xticklabel',{'8 Hz (8 Hz)','13 Hz (13 Hz)','19 Hz (19 Hz)','25 Hz (25 Hz)','8 Hz (16 Hz)','13 Hz (26 Hz)','19 Hz (38 Hz)', '25 Hz (50 Hz)'})
xlabel('Stimulus and response frequency')
ylabel('SNR')
errorbar([b(1).XEndPoints' b(2).XEndPoints' b(3).XEndPoints' b(4).XEndPoints' b(5).XEndPoints' b(6).XEndPoints' b(7).XEndPoints' b(8).XEndPoints' b(9).XEndPoints'],...
    squeeze(squeeze(mean(c,2,'omitnan'))'),[ci(1,:)' ci(2,:)' ci(3,:)' ci(4,:)' ci(5,:)' ci(6,:)' ci(7,:)' ci(8,:)' ci(9,:)'],'ko','Linewidth',1)
set(findall(gcf,'-property','FontName'),'FontName','Arial','Fontsize',32)
set(gca,'LooseInset', [0.01 0.01 0.01 0.01])
set(gca,'xlim',[.5 7.5])
set(gca,'ylim',[1 3.35])
grid on;ax=gca; ax.LineWidth=1.5;

% Experiment 2
ads2=[mean(SNR2(Conditions2(:,1)==1,1:7),2) mean(SNR2(Conditions2(:,1)==2,1:7),2) mean(SNR2(Conditions2(:,1)==3,1:7),2)]; % SNR for the three sizes
[~,~,adsc2]=ttest(ads2);
disp('Experiment 2: Mean SNR for the 3 sizes')
disp(round(mean(ads2),3))
disp('Experiment 2: 95% confidence intervals of SNRs for the 3 sizes')
disp(round(adsc2,3))

figure
set(gcf,'renderer','painters')
b=bar(squeeze(mean(c2,2,'omitnan'))');hold on
title('Experiment 2x2')
b(1).FaceColor=[0.9 0.9 0.9];
b(2).FaceColor=[0.7 0.7 0.7];
b(3).FaceColor=[0.5 0.5 0.5];
b(4).FaceColor=[0 1 1];
b(5).FaceColor=[127 255 212]/255;
b(6).FaceColor=[0 206 209]/255;
b(7).FaceColor=[0 150 150]/255;
b(8).FaceColor=[1 1 1 ];
b(9).FaceColor=[0 1 0 ];
b(10).FaceColor=[1 0 0 ];
legend('Small','Medium','Large','Left top','Right top','Left bottom','Right bottom','White','Green','Red','autoupdate','off','location','eastoutside');
set(gca,'xticklabel',{'8 Hz (8 Hz)','13 Hz (13 Hz)','19 Hz (19 Hz)','25 Hz (25 Hz)','8 Hz (16 Hz)','13 Hz (26 Hz)','19 Hz (38 Hz)', '25 Hz (50 Hz)'})
xlabel('Stimulus and response frequency')
ylabel('SNR')
errorbar([b(1).XEndPoints' b(2).XEndPoints' b(3).XEndPoints' b(4).XEndPoints' b(5).XEndPoints' b(6).XEndPoints' b(7).XEndPoints' b(8).XEndPoints' b(9).XEndPoints' b(10).XEndPoints'],...
    squeeze(squeeze(mean(c2,2,'omitnan'))'),[ci2(1,:)' ci2(2,:)' ci2(3,:)' ci2(4,:)' ci2(5,:)' ci2(6,:)' ci2(7,:)' ci2(8,:)' ci2(9,:)' ci2(10,:)'],'ko','Linewidth',1)
set(findall(gcf,'-property','FontName'),'FontName','Arial','Fontsize',32)
set(gca,'LooseInset', [0.01 0.01 0.01 0.01])
set(gca,'xlim',[.5 7.5])
set(gca,'ylim',[1 3.35])
grid on;ax=gca; ax.LineWidth=1.5;
%% Figure 7. Experiment 1: scatter plots
figure
set(gcf,'renderer','painters')
tiledlayout(2,4,'tilespacing','normal','padding','normal');
for ff=1:8
    if ff==1
        v1=SNR(Conditions(:,3)==1,2); % exp1, white, 13 Hz
        v2=SNR(Conditions(:,3)==3,2); % exp1, red, 13 Hz
    elseif ff==2
        v1=SNR(Conditions(:,3)==1,4); % exp1, white, 25 Hz
        v2=SNR(Conditions(:,3)==3,4); % exp1, red, 25 Hz
    elseif ff==3
        v1=SNR2(Conditions2(:,3)==1,2); % exp2, white, 13 Hz
        v2=SNR2(Conditions2(:,3)==3,2); % exp2, red, 13 Hz
    elseif ff==4
        v1=SNR2(Conditions2(:,3)==1,4); % exp2, white, 25 Hz
        v2=SNR2(Conditions2(:,3)==3,4); % exp2, red, 25 Hz
    elseif ff==5
        v1=SNR(Conditions(:,3)==1,2); % exp1, white, 13 Hz
        v2=SNR(Conditions(:,3)==2,2); % exp1, green, 13 Hz
    elseif ff==6
        v1=SNR(Conditions(:,3)==1,4); % exp1, white, 25 Hz
        v2=SNR(Conditions(:,3)==2,4); % exp1, green, 25 Hz
    elseif ff==7
        v1=SNR2(Conditions2(:,3)==1,2); % exp2, white, 13 Hz
        v2=SNR2(Conditions2(:,3)==2,2); % exp2, green, 13 Hz
    elseif ff==8
        v1=SNR2(Conditions2(:,3)==1,4); % exp2, white, 25 Hz
        v2=SNR2(Conditions2(:,3)==2,4); % exp2, green, 25 Hz
    end
    % surface area (10000, 20000, 30000)
    % shape index (square, circle, triangle)
    % color index (white, green, red)

    if ismember(ff,[1 2 5 6]) % Experiment 1
        sz = Conditions(Conditions(:,3)==1,1); % size of stimuli
        sh = Conditions(Conditions(:,3)==1,2); % shape of stimuli (1. square, 2. circle, 3. triangle)
    else % Experiment 2
        sz = Conditions2(Conditions2(:,3)==1,1); % size of stimuli
        sh = Conditions2(Conditions2(:,3)==1,2); % position of stimuli
        sh=sign(sh); % use all circles as shapes
    end
    nexttile
    for i = 1:9
        if i==1 % square
            scatter1 = scatter(mean(v1(sz==1 & sh==1)), mean(v2(sz==1 & sh==1)), 1*250, 's','markerfacecolor', 'k', 'markeredgecolor', 'k', 'linewidth',2);hold on
        elseif i==2 % square
            scatter1 = scatter(mean(v1(sz==2 & sh==1)), mean(v2(sz==2 & sh==1)), 2*250, 's','markerfacecolor', 'k', 'markeredgecolor', 'k', 'linewidth',2);
        elseif i==3 % square
            scatter1 = scatter(mean(v1(sz==3 & sh==1)), mean(v2(sz==3 & sh==1)), 3*250, 's','markerfacecolor', 'k', 'markeredgecolor', 'k', 'linewidth',2);
        end


        if i==4 % circle
            scatter1 = scatter(mean(v1(sz==1 & sh==2)), mean(v2(sz==1 & sh==2)), 1*250, 'o','markerfacecolor', 'k', 'markeredgecolor', 'k', 'linewidth',2);
        elseif i==5 % circle
            scatter1 = scatter(mean(v1(sz==2 & sh==2)), mean(v2(sz==2 & sh==2)), 2*250, 'o','markerfacecolor', 'k', 'markeredgecolor', 'k', 'linewidth',2);
        elseif i==6 % circle
            scatter1 = scatter(mean(v1(sz==3 & sh==2)), mean(v2(sz==3 & sh==2)), 3*250, 'o','markerfacecolor', 'k', 'markeredgecolor', 'k', 'linewidth',2);
        end

        if i==7 % triangle
            scatter1 = scatter(mean(v1(sz==1 & sh==3)), mean(v2(sz==1 & sh==3)), 1*250, '^','markerfacecolor', 'k', 'markeredgecolor', 'k', 'linewidth',2);
        elseif i==8 % triangle
            scatter1 = scatter(mean(v1(sz==2 & sh==3)), mean(v2(sz==2 & sh==3)), 2*250, '^','markerfacecolor', 'k', 'markeredgecolor', 'k', 'linewidth',2);
        elseif i==9 % triangle
            scatter1 = scatter(mean(v1(sz==3 & sh==3)), mean(v2(sz==3 & sh==3)), 3*250, '^','markerfacecolor', 'k', 'markeredgecolor', 'k', 'linewidth',2);
        end
        scatter1.MarkerFaceAlpha = 0.2;
    end
    hold on;
    grid on;ax=gca; ax.LineWidth=1.5;
    box on
    plot([0 4],[0 4],'k:','Linewidth',3)
    set(gca,'xlim',[1 3.8],'xtick',1:0.5:4)
    set(gca,'ylim',[1 3.8],'xtick',1:0.5:4)
    axis square

    if ff==1 || ff==5
        title('Experiment 1x1 – 13 Hz (13 Hz)')
    elseif ff==2 || ff==6
        title('Experiment 1x1 – 25 Hz (25 Hz)')
    elseif ff==3 || ff==7
        title('Experiment 2x2 – 13 Hz (13 Hz)')
    elseif ff==4 || ff==8
        title('Experiment 2x2 – 25 Hz (25 Hz)')
    end

    ax = gca;  % Get current axes
    if ismember(ff,[1 2 3 4]) % 13 Hz (Exp 1, Exp 2)
        xlabel('\color{gray}SNR – White', 'Interpreter', 'tex','fontweight','bold')
        ylabel('\color{red}SNR – Red', 'Interpreter', 'tex','fontweight','bold')
        ax.XColor = [0.5 0.5 0.5];   % set the x-axis color to grey
        ax.YColor = [1 0 0];  % set the y-axis color to red
    elseif ismember(ff,[5 6 7 8]) % 25 Hz (Exp 1, Exp 2)
        xlabel('SNR – White','fontsize', 30)
        xlabel('\color{gray}SNR – White', 'Interpreter', 'tex','fontweight','bold')
        ylabel('\color{darkgreen}SNR – Green', 'Interpreter', 'tex','fontweight','bold')
        ax.XColor = [0.5 0.5 0.5]; % set the x-axis color to grey
        ax.YColor = [105 149 58]/255; % set the y-axis color to dark green
    end
    ax.GridLineWidth = 1.5;
    set(findall(gcf,'-property','FontName'),'FontName','Arial','Fontsize',20)
    set(gca,'LooseInset', [0.01 0.01 0.01 0.01])
end


%% Figure 8: Experiment 1: Pupil diameter
figure
set(gcf,'renderer','painters')
subplot(1,3,1)

%i1=1:3 % block index (1, 2, or 3). A block consists of 108 trials
%i2=1:4 % frequencies
%i3=1:3 % surface area (10000, 20000, 30000)
%i4=1:3 % shape index (square, circle, triangle)
%i5=1:3 % color index (white, green, red)
time=0.0005:0.0005:size(PTSa,3)/2000;
subplot(1,4,1)
temp=squeeze(mean(PTSa(1:6,cdv(:,2)==1,:),1,'omitnan'));
plot(time,mean(temp),'color',[1 0.5 0],'Linewidth',3);hold on
temp=squeeze(mean(PTSa(1:6,cdv(:,2)==2,:),1,'omitnan'));
plot(time,mean(temp),'color',[0.5 1 0],'Linewidth',3);
temp=squeeze(mean(PTSa(1:6,cdv(:,2)==3,:),1,'omitnan'));
plot(time,mean(temp),'color',[0 0 1],'Linewidth',3);
temp=squeeze(mean(PTSa(1:6,cdv(:,2)==4,:),1,'omitnan'));
plot(time,mean(temp),'color',[0 0 0],'Linewidth',3);

set(gca,'ylim',[-5.5 1],'pos',[0.06 0.104 0.2 0.88],'xtick',0:.5:4)
grid on;ax=gca; ax.LineWidth=1.5;
legend('8 Hz','13 Hz','19 Hz','25 Hz')
xlabel('Time (s)')
ylabel('Pupil diameter change (%)')

subplot(1,4,2)
temp=squeeze(mean(PTSa(1:6,cdv(:,3)==1,:),1,'omitnan'));
plot(time,mean(temp),'color',[0.9 0.9 0.9],'Linewidth',3);hold on
temp=squeeze(mean(PTSa(1:6,cdv(:,3)==2,:),1,'omitnan'));
plot(time,mean(temp),'color',[0.7 0.7 0.7],'Linewidth',3);
temp=squeeze(mean(PTSa(1:6,cdv(:,3)==3,:),1,'omitnan'));
plot(time,mean(temp),'color',[0.5 0.5 0.5],'Linewidth',3);

set(gca,'ylim',[-5.5 1],'pos',[0.30 0.104 0.2 0.88],'YTickLabel',[],'xtick',0:.5:4)
grid on;ax=gca; ax.LineWidth=1.5;
legend('Small','Medium','Large')
xlabel('Time (s)')

subplot(1,4,3)
temp=squeeze(mean(PTSa(1:6,cdv(:,4)==1,:),1,'omitnan'));
plot(time,mean(temp),'color',[0 1 1],'Linewidth',3);hold on
temp=squeeze(mean(PTSa(1:6,cdv(:,4)==2,:),1,'omitnan'));
plot(time,mean(temp),'color',[127 255 212]/255,'Linewidth',3);
temp=squeeze(mean(PTSa(1:6,cdv(:,4)==3,:),1,'omitnan'));
plot(time,mean(temp),'color',[0 206 209]/255,'Linewidth',3);
set(gca,'ylim',[-5.5 1],'pos',[0.54 0.104 0.2 0.88],'YTickLabel',[],'xtick',0:.5:4)
grid on;ax=gca; ax.LineWidth=1.5;
legend('Square','Circle','Triangle')
xlabel('Time (s)')

subplot(1,4,4)
time=0.0005:0.0005:size(PTSa,3)/2000;
temp=squeeze(mean(PTSa(1:6,cdv(:,5)==1,:),1,'omitnan'));
plot(time,mean(temp),'color',[0.9 0.9 0.9],'Linewidth',3);hold on
temp=squeeze(mean(PTSa(1:6,cdv(:,5)==2,:),1,'omitnan'));
plot(time,mean(temp),'color',[0 1 0],'Linewidth',3);
temp=squeeze(mean(PTSa(1:6,cdv(:,5)==3,:),1,'omitnan'));
plot(time,mean(temp),'color',[1 0 0],'Linewidth',3);
set(gca,'ylim',[-5.5 1],'pos',[0.78 0.104 0.2 0.88],'YTickLabel',[],'xtick',0:.5:4)
grid on;ax=gca; ax.LineWidth=1.5;
legend('White','Green','Red')
xlabel('Time (s)')
set(findall(gcf,'-property','FontName'),'FontName','Arial','Fontsize',30)
set(gca,'LooseInset', [0.01 0.01 0.01 0.01])

B328=NaN(size(PTSa,2),1);

for i=1:size(PTSa,2)
    idb=find(Brightness(:,1)==cdv(i,2) & Brightness(:,2)==cdv(i,3) & Brightness(:,3)==cdv(i,4) & Brightness(:,4)==cdv(i,5));
    B328(i)=Brightness(idb,5); % Brightness of trials
end

%% Figure 9. Experiment 1: Pupil diameter bar graph
PD=mean(PTSa(1:6,:,2001),3,'omitnan')'; % PC1, pupil diameter change after 1 second
PS=NaN(81,4);
Conditions=cdv(cdv(:,2)==1,[3 4 5]); % surface area, shape, color for 1 repetition
for f=1:4 % loop over 4 frequencies
    for i=1:size(Conditions,1) % 81 conditions
        idx=find(cdv(:,2)==f & cdv(:,3)==Conditions(i,1) & cdv(:,4)==Conditions(i,2) & cdv(:,5)==Conditions(i,3));
        PS(i,f) = mean(mean(PD(idx,:),1,'omitnan')); % pupil size
    end
end

clear c
% 95% confidence intervals are calculated over 27 data values
[~,~,c(1,1:2,1:4)]=ttest(PS(Conditions(:,1)==1,:)); % surface area 10000
[~,~,c(2,1:2,1:4)]=ttest(PS(Conditions(:,1)==2,:)); % surface area 20000
[~,~,c(3,1:2,1:4)]=ttest(PS(Conditions(:,1)==3,:)); % surface area 30000
[~,~,c(4,1:2,1:4)]=ttest(PS(Conditions(:,2)==1,:)); % square
[~,~,c(5,1:2,1:4)]=ttest(PS(Conditions(:,2)==2,:)); % circle
[~,~,c(6,1:2,1:4)]=ttest(PS(Conditions(:,2)==3,:)); % triangle
[~,~,c(7,1:2,1:4)]=ttest(PS(Conditions(:,3)==1,:)); % white
[~,~,c(8,1:2,1:4)]=ttest(PS(Conditions(:,3)==2,:)); % green
[~,~,c(9,1:2,1:4)]=ttest(PS(Conditions(:,3)==3,:)); % red
ci=squeeze((c(:,2,:)-c(:,1,:))/2);

figure
set(gcf,'renderer','painters')
b=bar(squeeze(mean(c,2,'omitnan'))');hold on
title('Experiment 1x1')
b(1).FaceColor=[0.9 0.9 0.9];
b(2).FaceColor=[0.7 0.7 0.7];
b(3).FaceColor=[0.5 0.5 0.5];
b(4).FaceColor=[0 1 1];
b(5).FaceColor=[127 255 212]/255;
b(6).FaceColor=[0 206 209]/255;
b(7).FaceColor=[1 1 1];
b(8).FaceColor=[0 1 0];
b(9).FaceColor=[1 0 0];
grid on;ax=gca; ax.LineWidth=1.5;
legend('Small','Medium','Large','Square','Circle','Triangle','White','Green','Red','autoupdate','off')
set(gca,'xticklabel',{'8 Hz','13 Hz','19 Hz','25 Hz'})
xlabel('Stimulus frequency')
ylabel('Pupil diameter change after 1 second (%)')
errorbar([b(1).XEndPoints' b(2).XEndPoints' b(3).XEndPoints' b(4).XEndPoints' b(5).XEndPoints' b(6).XEndPoints' b(7).XEndPoints' b(8).XEndPoints' b(9).XEndPoints'],...
    squeeze(squeeze(mean(c,2,'omitnan'))'),[ci(1,:)' ci(2,:)' ci(3,:)' ci(4,:)' ci(5,:)' ci(6,:)' ci(7,:)' ci(8,:)' ci(9,:)'],'ko','Linewidth',1)
set(findall(gcf,'-property','FontName'),'FontName','Arial','Fontsize',30)
set(gca,'LooseInset', [0.01 0.01 0.01 0.01])

%% Figure 10. Experiment 1: scatter plot Brightness and mean pupil diameter
BRS = NaN(3,3);
PS = NaN(3,3);
figure
set(gcf,'renderer','painters')
for i3=1:3 % size
    for i5=1:3 % color
        idx=find(cdv(:,3)==i3 & cdv(:,5)==i5); % surface area, color (3 repetitions x 4 flicker frequencies x 3 shapes)
        BRS(i3,i5) = mean(B328(idx)); % brightness for scatter plot
        PS(i3,i5) = mean(mean(PD(idx,:),1,'omitnan')); % pupil size
        %  PS(i3,i5) = median(median(PD(idx,:),1,'omitnan')); % pupil size
        if i5==1
            scatter(BRS(i3,i5), PS(i3,i5), i3*500, 'o','markerfacecolor', 'w', 'markeredgecolor', 'k', 'linewidth',2);hold on
        elseif i5==2
            scatter(BRS(i3,i5), PS(i3,i5), i3*500, 'o','markerfacecolor', 'g', 'markeredgecolor', 'k', 'linewidth',2);hold on
        elseif i5==3
            scatter(BRS(i3,i5), PS(i3,i5), i3*500, 'o','markerfacecolor', 'r', 'markeredgecolor', 'k', 'linewidth',2);hold on
        end
    end
end
title('Experiment 1x1')
grid on;ax=gca; ax.LineWidth=1.5;
xlabel('Brightness score')
ylabel('Pupil diameter change after 1 second (%)')
set(findall(gcf,'-property','FontName'),'FontName','Arial','Fontsize',30)
set(gca,'LooseInset', [0.01 0.01 0.01 0.01])
set(gcf,'Pos',[707 88 988 944])

%% Figure S1 , showing the Baseline FFT of the 6 individual participants in Experiment 1
f = Fs*(0:(L/2))/L; % vector of frequencies (0, 0.25, 0.50, ... 1249.75, 1250)

figure
set(gcf,'renderer','painters')
plot(f(1:size(fft_exp1_baseline,2)),fft_exp1_baseline,'Linewidth',4);hold on

set(gca,'xlim',[1 76],'ylim',[0 3.2],'xtick',0:2:100)
grid on;ax=gca; ax.LineWidth=1.5;
legend('Participant 1','Participant 2','Participant 3','Participant 4','Participant 5','Participant 6')
title('Experiment 1x1')
xlabel('Frequency (Hz)')
ylabel('Magnitude')
set(findall(gcf,'-property','FontName'),'FontName','Arial','Fontsize',30)
set(gca,'LooseInset', [0.01 0.01 0.01 0.01])

%% Figure S2, showing the Baseline FFT of the 6 individual participants in Experiment 2
f = Fs*(0:(L/2))/L; % vector of frequencies (0, 0.25, 0.50, ... 1249.75, 1250)

figure
set(gcf,'renderer','painters')
plot(f(1:size(fft_exp2_baseline,2)),fft_exp2_baseline,'Linewidth',4);hold on

set(gca,'xlim',[1 76],'ylim',[0 3.2],'xtick',0:2:100)
grid on;ax=gca; ax.LineWidth=1.5;
legend('Participant 1','Participant 2','Participant 3','Participant 4','Participant 5','Participant 6')
title('Experiment 2x2')
xlabel('Frequency (Hz)')
ylabel('Magnitude')
set(findall(gcf,'-property','FontName'),'FontName','Arial','Fontsize',30)
set(gca,'LooseInset', [0.01 0.01 0.01 0.01])

%% Figure S3 and S4: bar graphs of classification results
Classification1M=squeeze(mean(Classification1,2))';
clear ccd1
[~,~,cc1(1,1:2,1:4)]=ttest(Classification1M(Conditions(:,1)==1,:)); % size
[~,~,cc1(2,1:2,1:4)]=ttest(Classification1M(Conditions(:,1)==2,:));
[~,~,cc1(3,1:2,1:4)]=ttest(Classification1M(Conditions(:,1)==3,:));
[~,~,cc1(4,1:2,1:4)]=ttest(Classification1M(Conditions(:,2)==1,:)); % shape
[~,~,cc1(5,1:2,1:4)]=ttest(Classification1M(Conditions(:,2)==2,:));
[~,~,cc1(6,1:2,1:4)]=ttest(Classification1M(Conditions(:,2)==3,:));
[~,~,cc1(7,1:2,1:4)]=ttest(Classification1M(Conditions(:,3)==1,:)); % color
[~,~,cc1(8,1:2,1:4)]=ttest(Classification1M(Conditions(:,3)==2,:));
[~,~,cc1(9,1:2,1:4)]=ttest(Classification1M(Conditions(:,3)==3,:));

figure
set(gcf,'renderer','painters')
b=bar(squeeze(mean(cc1,2,'omitnan'))','Linewidth',2);hold on
title('Experiment 1x1')
b(1).FaceColor=[0.9 0.9 0.9];
b(2).FaceColor=[0.7 0.7 0.7];
b(3).FaceColor=[0.5 0.5 0.5];
b(4).FaceColor=[0 1 1];
b(5).FaceColor=[127 255 212]/255;
b(6).FaceColor=[0 206 209]/255;
b(7).FaceColor=[1 1 1 ];
b(8).FaceColor=[0 1 0 ];
b(9).FaceColor=[1 0 0 ];
legend('Small','Medium','Large','Square','Circle','Triangle','White','Green','Red','autoupdate','off','location','eastoutside');
set(gca,'xticklabel',{'8 Hz','13 Hz','19 Hz','25 Hz'})
xlabel('Stimulus and response frequency')
ylabel('Proportion correct')
set(findall(gcf,'-property','FontName'),'FontName','Arial','Fontsize',28)
set(gca,'LooseInset', [0.01 0.01 0.01 0.01])
set(gca,'xlim',[.5 4.5])
set(gca,'ylim',[0.5 0.9])
grid on;ax=gca; ax.LineWidth=1.5;
% Experiment 2
Classification2M=squeeze(mean(Classification2,2))';
clear cc2
[~,~,cc2(1,1:2,1:4)]=ttest(Classification2M(Conditions2(:,1)==1,:));
[~,~,cc2(2,1:2,1:4)]=ttest(Classification2M(Conditions2(:,1)==2,:));
[~,~,cc2(3,1:2,1:4)]=ttest(Classification2M(Conditions2(:,1)==3,:));
[~,~,cc2(4,1:2,1:4)]=ttest(Classification2M(Conditions2(:,2)==1,:));
[~,~,cc2(5,1:2,1:4)]=ttest(Classification2M(Conditions2(:,2)==2,:));
[~,~,cc2(6,1:2,1:4)]=ttest(Classification2M(Conditions2(:,2)==3,:));
[~,~,cc2(7,1:2,1:4)]=ttest(Classification2M(Conditions2(:,2)==4,:));
[~,~,cc2(8,1:2,1:4)]=ttest(Classification2M(Conditions2(:,3)==1,:));
[~,~,cc2(9,1:2,1:4)]=ttest(Classification2M(Conditions2(:,3)==2,:));
[~,~,cc2(10,1:2,1:4)]=ttest(Classification2M(Conditions2(:,3)==3,:));

figure
set(gcf,'renderer','painters')
b=bar(squeeze(mean(cc2,2,'omitnan'))','linewidth',2);hold on
title('Experiment 2x2')
b(1).FaceColor=[0.9 0.9 0.9];
b(2).FaceColor=[0.7 0.7 0.7];
b(3).FaceColor=[0.5 0.5 0.5];
b(4).FaceColor=[0 1 1];
b(5).FaceColor=[127 255 212]/255;
b(6).FaceColor=[0 206 209]/255;
b(7).FaceColor=[0 150 150]/255;
b(8).FaceColor=[1 1 1 ];
b(9).FaceColor=[0 1 0 ];
b(10).FaceColor=[1 0 0 ];
legend('Small','Medium','Large','Left top','Right top','Left bottom','Right bottom','White','Green','Red','autoupdate','off','location','eastoutside');
set(gca,'xticklabel',{'8 Hz','13 Hz','19 Hz','25 Hz'})
xlabel('Stimulus and response frequency')
ylabel('Proportion correct')
set(findall(gcf,'-property','FontName'),'FontName','Arial','Fontsize',28)
set(gca,'LooseInset', [0.01 0.01 0.01 0.01])
set(gca,'xlim',[.5 4.5])
set(gca,'ylim',[0.5 0.9])
grid on;ax=gca; ax.LineWidth=1.5;
